<?php

/**
 * Specify the default properties of a uc_attribute.
 *
 * @return
 *   An array defining the default structure of a uc_attribute.
 */
function _webform_defaults_uc_attribute() {
  return array(
    'name' => '',
    'form_key' => NULL,
    'mandatory' => 0,
    'pid' => 0,
    'weight' => 0,
    'extra' => array(
      'width' => '',
      'aid' => '',
      'title_display' => 0,
      'description' => '',
      'private' => 0,
    ),
  );
}

/**
 * Generate the form for editing a uc_attribute.
 *
 * Create a set of form elements to be displayed on the form for editing this
 * uc_attribute. Use care naming the form items, as this correlates directly to
 * the database schema. The uc_attribute "Name" and "Description" fields are
 * added to every uc_attribute type and are not necessary to specify here
 * (although they may be overridden if desired).
 *
 * @todo Add JS to update the label and description when the AID changes, and
 *    to add/remove the width textfield.
 *
 * @param $uc_attribute
 *   A Webform uc_attribute array.
 * @return
 *   An array of form items to be displayed on the edit uc_attribute page
 */
function _webform_edit_uc_attribute($component) {
  $form = array();

  $nid = $component['nid'];
  // The AID will be the empty string if this is a new component.
  $aid = $component['extra']['aid'];
  $options = _uc_event_registration_get_remainining_attributes($nid, $aid);

  // If there aren't any attributes associated with this webform, or if they
  // all already have components then display a message and pass an empty form,
  // using an empty markup element to stop webform from complaining the
  // component doesn't have an edit function defined.
  if (empty($options)) {
    drupal_set_message(t('There are no (remaining) product attributes associated with this node.'));
    $form['uc_attribute'] = array('#type' => 'markup', '#value' => '');
    return $form;
  }

  // Options are stored in the "extra" array, which stores any settings unique
  // to uc_attribute.
  $form['extra']['aid'] = array(
    '#type' => 'select',
    '#title' => t('Product attribute'),
    '#options' => $options,
    '#default_value' => $aid,
    '#description' => t('Select product attribute to display for this form element'),
    '#weight' => -3,
    '#required' => TRUE,
  );

  $form['display']['width'] = array(
    '#type' => 'textfield',
    '#title' => t('Width'),
    '#default_value' => $component['extra']['width'],
    '#description' => t('If the attribute is a textfield, then set its width here.') . ' ' . t('Leaving blank will use the default size.'),
    '#size' => 5,
    '#maxlength' => 10,
    '#weight' => 0,
    '#parents' => array('extra', 'width'),
  );

  $attribute = _webform_uc_attribute_get_attribute($component);

  // Set the component name from the attribute if possible. The description
  // is not stored in the component, but read from the actual attribute.
  $label = $attribute ? $attribute->label : t("Product attribute");
  $form['name'] = array('#type' => 'value', '#value' => $label);
  $form['extra']['description'] = array('#type' => 'value', '#value'=>'');

  $form['uc_attribute_name'] = array(
    '#type' => 'item',
    '#title' => t('Label'),
    '#value' => $attribute ? $attribute->label : t("No attribute selected"),
    '#description' => t("The label is set on the <a href=\"!attribute_url\">attribute screen</a>.", array('!attribute_url' => url("node/$nid/edit/attributes"))),
    '#weight' => -9,
  );
  // If there's no attribute set (new component) then link to the general
  // attributes page.
  $attr_url = 'admin/store/attributes';
  if ($attribute) {
    $attr_url .= "/$aid/edit";
  }
  $form['uc_attribute_description'] = array(
    '#type' => 'item',
    '#title' => t('Description'),
    '#value' => $attribute ? $attribute->description : t("No attribute selected"),
    '#description' => t("The description is set on <a href=\"!edit_attribute_url\">the <em>Edit attribute</em> page</a>.", array('!edit_attribute_url' => url($attr_url))),
  );
  return $form;
}

/**
 * Render a Webform uc_attribute to be part of a form.
 *
 * @param $uc_attribute
 *   A Webform uc_attribute array.
 * @param $value
 *   If editing an existing submission or resuming a draft, this will contain
 *   an array of values to be shown instead of the default in the uc_attribute
 *   configuration. This value will always be an array, keyed numerically for
 *   each value saved in this field.
 * @param $filter
 *   Whether or not to filter the contents of descriptions and values when
 *   rendering the uc_attribute. Values need to be unfiltered to be editable by
 *   Form Builder.
 *
 * @see _webform_client_form_add_uc_attribute()
 */
function _webform_render_uc_attribute($component, $value = NULL, $filter = true) {

  $nid = $component['nid'];

  // Make sure the attribute is valid for this product. If not, display nothing,
  // which is done via an empty markup element to avoid webform complaining that
  // the component is not able to be displayed.
  $attribute = _webform_uc_attribute_get_attribute($component);
  if (!$attribute) {
    return array('#type' => 'markup', '#value' => '');
  }

  _webform_uc_attribute_update_from_attribute($component);

  // If this component is a text field, use textfield render callback
  $is_textfield = _webform_uc_attribute_is_textfield($component);
  if ($is_textfield) {
    // Use the default textfield 'extra' array to make a valid textfield
    // component.
    $textfield_defaults = _webform_defaults_textfield();
    $component['extra'] += $textfield_defaults['extra'];
    $component['type'] = 'textfield';
    return webform_component_invoke ('textfield', 'render', $component, $value, $filter);
  }


  // The webform type is determined by the product attribute display type
  switch ($attribute->display) {
    case 1:
     $attr_type = 'select';
     break;
    case 2:
     $attr_type = 'radios';
     break;
    case 3:
     $attr_type = 'checkboxes';
     break;
    }

  // Build up the display of the option name (including price) (taken from
  // uc_attribute.module _uc_attribute_alter_form).
  $context = array(
    'revision' => 'formatted',
  );
  $options = array();
  $qty = 1;

  $product = node_load($nid);
  $priced_attributes = uc_attribute_priced_attributes($nid);

  foreach ($attribute->options as $option) {
    $context['subject']['option'] = $option;
    switch (variable_get('uc_attribute_option_price_format', 'adjustment')) {
      case 'total':
        $context['type'] = 'product';
        $context['subject']['node'] = $product;
        $price_info = array(
          'price' => $product->sell_price + $option->price,
          'qty' => $qty,
         );
         $display_price = in_array($attribute->aid, $priced_attributes) ? ', '. uc_price($price_info, $context) : '';
         if (count($priced_attributes) == 1 && $attribute->display != 3) {
           break;
         }
       case 'adjustment':
         $context['type'] = 'attribute_option';
         $price_info = array(
           'price' => $option->price,
           'qty' => $qty,
         );
         $display_price = ($option->price != 0 ? ', '. ($option->price > 0 ? '+' : '') . uc_price($price_info, $context) : '');
         break;
      case 'none':
        default:
          $display_price = '';
          break;
    }
    $options[$option->oid] = $option->name;
    if ($attribute->display == 2) {
      // Select options are check_plain()ed, but radio button labels are not.
      $options[$option->oid] = check_plain($options[$option->oid]);
    }
    $options[$option->oid] .= $display_price;
  }

  // Add a "please select" option for attributes that are displaying as select lists
  if (count($attribute->options)) {
      if ($attribute->display == 1) {
        $options = array('' => t('Please select')) + $options;
      }
  }

  $form_item = array(
    '#type' => $attr_type,
    '#options' => $options,
    '#title' => $filter ? _webform_filter_xss($component['name']) : $component['name'],
    '#title_display' => $component['extra']['title_display'] ? $component['extra']['title_display'] : 'before',
    '#required' => $component['mandatory'],
    '#weight' => $component['weight'],
    '#description'   => $filter ? _webform_filter_descriptions($component['extra']['description']) : $component['extra']['description'],
    '#theme_wrappers' => array('webform_element_wrapper'),
    '#pre_render' => array('webform_element_title_display'),
    '#post_render' => array('webform_element_wrapper'),
  );

  if (isset($component['value']) && !empty($component['value'])) {
    $form_item['#default_value'] = $filter ? _webform_filter_values($component['value']) : $component['value'];
  }

  //  If the value is set, we are probably editing a submission
  //  - if radio buttons, set first element in array otherwise assign the array to the default_value
  if (isset($value)) {
     if ($attribute->display == 2) {
        $form_item['#default_value'] = $value[0];
     }
     else {
        $form_item['#default_value'] = $value;
     }
  }

  return $form_item;
}

/**
 * Display the result of a submission for a uc_attribute.
 *
 * The output of this function will be displayed under the "Results" tab then
 * "Submissions". This should output the saved data in some reasonable manner.
 *
 * @param $uc_attribute
 *   A Webform uc_attribute array.
 * @param $value
 *   An array of information containing the submission result, directly
 *   correlating to the webform_submitted_data database table schema.
 * @param $format
 *   Either 'html' or 'text'. Defines the format that the content should be
 *   returned as. Make sure that returned content is run through check_plain()
 *   or other filtering functions when returning HTML.
 * @return
 *   A renderable element containing at the very least these properties:
 *    - #title
 *    - #weight
 *    - #uc_attribute
 *    - #format
 *    - #value
 *   Webform also uses #theme_wrappers to output the end result to the user,
 *   which will properly format the label and content for use within an e-mail
 *   (such as wrapping the text) or as HTML (ensuring consistent output).
 */
function _webform_display_uc_attribute($uc_attribute, $value, $format = 'html') {

  _webform_uc_attribute_update_from_attribute($uc_attribute);

  // If this component is a text field, use textfield display callback
  $is_textfield = _webform_uc_attribute_is_textfield($uc_attribute);
  if ($is_textfield) {
    $uc_attribute['extra']['field_prefix'] = '';
    $uc_attribute['extra']['field_suffix'] = '';
    return webform_component_invoke ('textfield', 'display', $uc_attribute, $value, $format);
  }

  $titles = array();
  if (isset($value[0])) {
    $options = _webform_uc_attribute_get_attribute_options($uc_attribute, TRUE);
    
    foreach ($value as $key => $optvalue) {
      if (isset($options[$optvalue]['title'])) {
        $titles[] = $options[$optvalue]['title'];
      }
    }
  }
  $display = implode(', ', $titles);

  return array(
    '#title' => $uc_attribute['name'],
    '#weight' => $uc_attribute['weight'],
    '#theme' => 'webform_display_textfield',
    '#theme_wrappers' => $format == 'html' ? array('webform_element') : array('webform_element_text'),
    '#post_render' => array('webform_element_wrapper'),
    '#field_prefix' => '',
    '#field_suffix' => '',
    '#uc_attribute' => $uc_attribute,
    '#format' => $format,
    '#value' => $display,
    '#webform_component' => $uc_attribute,
  );
}

/**
 * A hook for changing the input values before saving to the database.
 *
 * Webform expects a uc_attribute to consist of a single field, or a single array
 * of fields. If you have a uc_attribute that requires a deeper form tree
 * you must flatten the data into a single array using this callback
 * or by setting #parents on each field to avoid data loss and/or unexpected
 * behavior.
 *
 * Note that Webform will save the result of this function directly into the
 * database.
 *
 * @param $uc_attribute
 *   A Webform uc_attribute array.
 * @param $value
 *   The POST data associated with the user input.
 * @return
 *   An array of values to be saved into the database. Note that this should be
 *   a numerically keyed array.
 */
function _webform_submit_uc_attribute($component, $value) {
  $return = NULL;
  if (is_array($value)) {
    // Build a list of all valid keys expected to be submitted.
    $attribute = _webform_uc_attribute_get_attribute ($component);
    $options = _webform_uc_attribute_get_attribute_options($component, TRUE);
    $return = array();
    foreach ($value as $key => $option_value) {
      // Handle options that are specified options.
      if ($option_value !== '' && isset($options[$option_value])) {
        // Checkboxes submit a value of FALSE when unchecked. A checkbox with
        // a value of '0' is valid, so we can't use empty() here.
        if ($option_value === FALSE && $attribute[display] == 3) {
          unset($value[$option_value]);
        }
        else {
          $return[] = $option_value;
        }
      }
    }
  }
  elseif (is_string($value)) {
    $return = $value;
  }

  return $return;
}

/**
 * Module specific instance of hook_help().
 *
 * This allows each Webform uc_attribute to add information into hook_help().
 */
function _webform_help_uc_attribute($section) {
  switch ($section) {
    case 'admin/settings/webform#uc_attribute_description':
      return t('Allows Ubercart Product attributes to be presented as webform questions which could affect the price of the product purchased.');
  }
}

/**
 * Calculate and returns statistics about results for this component.
 *
 * This takes into account all submissions to this webform. The output of this
 * function will be displayed under the "Results" tab then "Analysis".
 *
 * @param $uc_attribute
 *   An array of information describing the component, directly correlating to
 *   the webform_component database schema.
 * @param $sids
 *   An optional array of submission IDs (sid). If supplied, the analysis will
 *   be limited to these sids.
 * @param $single
 *   Boolean flag determining if the details about a single component are being
 *   shown. May be used to provided detailed information about a single
 *   component's analysis, such as showing "Other" options within a select list.
 * @return
 *   An array of data rows, each containing a statistic for this component's
 *   submissions.
 */
function _webform_analysis_uc_attribute($uc_attribute, $sids = array(), $single = FALSE) {
  // If this component is a text field, use textfield analysis callback
  $is_textfield = _webform_uc_attribute_is_textfield($uc_attribute);
  if ($is_textfield) {
    return webform_component_invoke ('textfield', 'analysis', $uc_attribute, $sids, $single);
  }

  $options = _webform_uc_attribute_get_attribute_options ($uc_attribute);

  // Generate a lookup table of results.
  $sidfilter = count($sids) ? " AND sid in (" . db_placeholders($sids, 'int') . ")" : "";
  $query = 'SELECT data, count(data) as datacount '.
    ' FROM {webform_submitted_data} '.
    ' WHERE nid = %d '.
    ' AND cid = %d '.
    " AND data != '' ". $sidfilter .
    ' GROUP BY data';
  $result = db_query($query, array_merge(array($uc_attribute['nid'], $uc_attribute['cid']), $sids));
  $counts = array();
  while ($data = db_fetch_object($result)) {
    $counts[$data->data] = $data->datacount;
  }

  $rows = array();
  foreach ($options as $oid => $option) {
    $rows[] = array($option['title'],!empty($counts[$oid]) ? $counts[$oid] : 0);
  }

  return $rows;
}

/**
 * Return the result of a uc_attribute value for display in a table.
 *
 * The output of this function will be displayed under the "Results" tab then
 * "Table".
 *
 * @param $uc_attribute
 *   A Webform uc_attribute array.
 * @param $value
 *   An array of information containing the submission result, directly
 *   correlating to the webform_submitted_data database schema.
 * @return
 *   Textual output formatted for human reading.
 */
function _webform_table_uc_attribute($uc_attribute, $value) {
  // If this component is a text field, return textfield table callback
  $is_textfield = _webform_uc_attribute_is_textfield($uc_attribute);
  if ($is_textfield) {
    return webform_component_invoke ('textfield', 'table', $uc_attribute, $value);
  }

  $options = _webform_uc_attribute_get_attribute_options ($uc_attribute);

  $output = '';
  // Return the value as a single string.
  if (is_array($value)) {
    foreach ($value as $item => $value) {
      if ($value !== '') {
        $output .= check_plain($options[$value]['title']) . '<br />';
      }
    }
  }
  else {
    $output = check_plain($value);
  }
  return $output;
}

/**
 * Return the header for this uc_attribute to be displayed in a CSV file.
 *
 * The output of this function will be displayed under the "Results" tab then
 * "Download".
 *
 * @param $uc_attribute
 *   A Webform uc_attribute array.
 * @param $export_options
 *   An array of options that may configure export of this field.
 *    //  $export_options['select_keys'] = 0/1 ; 0=Human readable;1=keys
 *    //  $export_options['select_format'] = separate/compact
 * @return
 *   An array of data to be displayed in the first three rows of a CSV file, not
 *   including either prefixed or trailing commas.
 */
function _webform_csv_headers_uc_attribute($uc_attribute, $export_options) {

  _webform_uc_attribute_update_from_attribute($uc_attribute);

  $header = array();
  // If this is compact format or a text field, just display the element name in the column header
  if ($export_options['select_format'] == 'compact' || _webform_uc_attribute_is_textfield($uc_attribute)) {
      $header[0] = array('');
      $header[1] = array('');
      $header[2] = array($uc_attribute['name']);
      return $header;
  }

  // Else, this is separate format - each attribute option is a new column
  $header[0] = array('');
  $header[1] = array($uc_attribute['name']);
  $options = _webform_uc_attribute_get_attribute_options($uc_attribute);

  $count = 0;
  foreach ($options as $key => $item) {
    // Empty column per sub-field in main header.
    if ($count != 0) {
      $header[0][] = '';
      $header[1][] = '';
    }
    // Depending on the select_keys export option, display either the option value or the option key.
    $header[2][] = ($export_options['select_keys'] == 0) ? $item['title'] : $key;

    $count++;
  }

  return $header;
}

/**
 * Format the submitted data of a uc_attribute for CSV downloading.
 *
 * The output of this function will be displayed under the "Results" tab then
 * "Download".
 *
 * @param $uc_attribute
 *   A Webform uc_attribute array.
 * @param $export_options
 *   An array of options that may configure export of this field.
 *    //  $export_options['select_keys'] = 0/1 ; 0=Human readable;1=keys
 *    //  $export_options['select_format'] = separate/compact
 * @param $value
 *   An array of information containing the submission result, directly
 *   correlating to the webform_submitted_data database schema.
 * @return
 *   An array of items to be added to the CSV file. Each value within the array
 *   will be another column within the file. This function is called once for
 *   every row of data.
 */
function _webform_csv_data_uc_attribute($uc_attribute, $export_options, $value) {
  // If this is a text field, just return the single value as provided
  if (_webform_uc_attribute_is_textfield($uc_attribute)) {
    return $value;
  }

  // If the value isn't an array (perhaps blank from an optional field), convert it to an array
  if (!is_array($value)) {
    $value = (array) $value;
  }

  $options = _webform_uc_attribute_get_attribute_options($uc_attribute);
  // If this is a compact format, return all the values in one cell...
  if ($export_options['select_format'] == 'compact') {
    $cell = '';
    foreach ($value as $key => $val) {
      if (!empty($cell)) {
        $cell .= ',';
      }
      $cell .= ($export_options['select_keys'] == 0) ? $options[$val]['title'] : $val;
    }
    return $cell;
  }

  // .. if this is separate format, return each value in a different cell.
  $return = array();
  $values = (is_array($value)) ? array_values($value) : array(); // If value is not an array, treat as an empty array
  foreach ($options as $oid => $option) {
    $return[] = in_array($oid, $values) ? 'X' : '';
  }
  return $return;
}

/**
 * Helper function to return an ubercart attribute for this component. It will
 * only search within the passed node, or one obtained via menu_get_object.
 *
 * @param $component
 *   The component whose attribute you want
 * @param $node
 *  (Optional) Node object to search for the attribute
 *
 * @return
 *   Attribute object if found; FALSE otherwise.
 *
 * @see uc_attribute_load()
 */
function _webform_uc_attribute_get_attribute($component, $node = NULL) {

  $nid = $component['nid'];
  $aid = $component['extra']['aid'];

  $attribute = FALSE;

  if ($nid && $aid) {

    if (!$node) {

      // We might be on a node or webform path, so try to get a node object from
      // one of those.
      $node = menu_get_object('webform_menu');

      if (!$node) {
        $node = menu_get_object();
      }
    }
    if ($node && ($node->nid == $nid) && isset($node->attributes[$aid])) {
      $attribute = $node->attributes[$aid];
    }
  }

  return $attribute;
}

/**
 * Helper function to return an array of valid attribute options for this
 * component.
 *
 * @param $component
 */
function _webform_uc_attribute_get_attribute_options ($component) {

  $attribute = _webform_uc_attribute_get_attribute($component);
  $options = array();

  foreach ($attribute->options as $option) {
    $options[$option->oid] = array('title'=>$option->name);
  }

  return $options;
}

/**
 * Updates the component with details from the attribute, and optionally updates
 * the component data in the DB.
 *
 * @param &$component
 *   The component to update
 * @param $update_db
 *   Whether to also update the component details in the database; this only
 *   updates the label.
 */
function _webform_uc_attribute_update_from_attribute(&$component, $update_db = FALSE) {

  $update_component_name_sql = "
  UPDATE {webform_component}
    SET
      name = '%s'
    WHERE
      nid = %d AND
      cid = %d
  ";

  $attribute = _webform_uc_attribute_get_attribute($component);
  if ($attribute) {
    if ($update_db && $component['name'] != $attribute->label) {
      // There's no need to update the description, as that's only used when
      // displaying the form element.
      db_query($update_component_name_sql, $attribute->label, $component['nid'], $component['cid']);
    }

    // Update the component's details.
    $component['name'] = $attribute->label;
    $component['extra']['description'] = $attribute->description;
  }
}

/**
 * Helper function to return whether the component should be treated
 * as a text field (ie. there are no options).
 * @param unknown_type $component
 */
function _webform_uc_attribute_is_textfield ($component) {
  $attribute = _webform_uc_attribute_get_attribute($component);
  if ($attribute->display == 0) {
    return true;
  }
  else {
    return false;
  }
}
